program test_mean_speed

  use, intrinsic:: ISO_FORTRAN_ENV

  ! Libraries:
  use contour_531, only: polyline
  use netcdf, only: nf90_nowrite
  use netcdf95, only: nf95_open, nf95_close, nf95_inq_varid, nf95_get_var, &
       find_coord, nf95_inquire_dimension
  use nr_util, only: pi
  use shapelib, only: shpclose, shpfileobject, shpobject
  use shapelib_03, only: shp_open_03, shp_read_object_03
  
  use mean_speed_m, only: mean_speed

  implicit none

  real m, corner(2)
  real:: center(2) = [9.625, - 33.875]
  integer ncid, varid, dimid
  integer nlon, nlat
  real, allocatable:: u(:, :), v(:, :) ! (nlon, nlat) wind, in m s-1
  TYPE(shpfileobject) hshp
  TYPE(shpobject) psobject
  type(polyline) p
  real, parameter:: deg_over_rad = pi / 180.

  namelist /main_nml/ center

  !---------------------------------------------------------------------

  write(unit = error_unit, nml = main_nml)
  write(unit = error_unit, fmt = *) "Enter namelist main_nml."
  read(unit = *, nml = main_nml)
  write(unit = *, nml = main_nml)
  
  call nf95_open("uv.nc", nf90_nowrite, ncid)
  
  call find_coord(ncid, dimid = dimid, varid = varid, std_name = "longitude")
  call nf95_inquire_dimension(ncid, dimid, nclen = nlon)
  call nf95_get_var(ncid, varid, corner(1))

  call find_coord(ncid, dimid = dimid, varid = varid, std_name = "latitude")
  call nf95_inquire_dimension(ncid, dimid, nclen = nlat)
  call nf95_get_var(ncid, varid, corner(2))

  allocate(u(nlon, nlat), v(nlon, nlat))
  
  call nf95_inq_varid(ncid, "ugos", varid)
  call nf95_get_var(ncid, varid, u)

  call nf95_inq_varid(ncid, "vgos", varid)
  call nf95_get_var(ncid, varid, v)

  call nf95_close(ncid)

  call shp_open_03("contour", "rb", hshp)
  call shp_read_object_03(hshp, 0, psobject)
  CALL shpclose(hshp)

  p%n_points = psobject%nvertices
  p%closed = .true.
  allocate(p%points(2, p%n_points))
  p%points(1, :) = psobject%padfx * deg_over_rad
  p%points(2, :) = psobject%padfy * deg_over_rad
  
  m = mean_speed(u, v, p, center * deg_over_rad, &
       corner * deg_over_rad, step = [0.25, 0.25]  * deg_over_rad)
  print *, "mean azimuthal speed = ", m, "m s-1"

end program test_mean_speed
