program test_set_all_outerm

  use, intrinsic:: ISO_FORTRAN_ENV

  ! Libraries:
  use contour_531, only: null_polyline
  use jumble, only: new_unit, get_command_arg_dyn
  use netcdf, only: nf90_nowrite
  use netcdf95, only: nf95_open, nf95_close, nf95_inq_varid, nf95_get_var, &
       find_coord, nf95_inquire_dimension, nf95_get_att
  use nr_util, only: pi, assert, deg_to_rad, rad_to_deg, twopi
  use shapelib, only: shpt_point, shpt_polygon, shpfileobject, ftdouble, &
       shpclose, ftinteger
  use shapelib_03, only: shp_create_03, dbf_add_field_03, shp_append_point_03, &
       dbf_write_attribute_03, shp_append_null_03, shp_append_object_03
  
  use derived_types, only: snapshot
  use get_var_m, only: get_var
  use set_all_outerm_m, only: set_all_outerm

  implicit none

  type(snapshot) s

  real:: min_amp = 0.
  ! minimum amplitude of ssh, between extremum and outermost contour,
  ! in m

  real:: min_radius = 20.
  ! minimum radius of the equal-area disk for of an outermost contour, in km
  
  character(len = :), allocatable:: filename
  integer nlon, nlat, copy
  real lon_min, lon_max, lat_min, lat_max ! longitudes and latitudes, in degrees

  integer:: max_radius(2) = [20, 20]
  ! maximum radius of an eddy in longitude and latitude, in number of
  ! grid points

  integer ncid, varid, dimid, i
  real, allocatable:: ssh(:, :)
  ! (1 - max_radius(1):nlon + max_radius(1), nlat) if the grid is periodic
  ! in longitude, else (nlon, nlat). Sea-surface height, in m.

  TYPE(shpfileobject) hshp_extremum ! shapefile extremum_$m
  TYPE(shpfileobject) hshp_outermost ! shapefile outermost_contour_$m
  integer ifield, ishape
  real step(2) ! longitude and latitude steps, in rad
  logical periodic ! grid is periodic in longitude
  namelist /main_nml/ min_amp, max_radius, min_radius

  !--------------------------------------------------------------

  call get_command_arg_dyn(1, filename, &
       "Required argument: NetCDF file containing adt")

  write(unit = error_unit, nml = main_nml)
  write(unit = error_unit, fmt = *) "Enter namelist main_nml."
  read(unit = *, nml = main_nml)
  write(unit = *, nml = main_nml)

  ! Read ssh:

  call nf95_open(filename, nf90_nowrite, ncid)

  call find_coord(ncid, dimid = dimid, varid = varid, std_name = "longitude")
  call nf95_inquire_dimension(ncid, dimid, nclen = nlon)
  call nf95_get_var(ncid, varid, lon_min)
  call nf95_get_var(ncid, varid, lon_max, start = [nlon])

  call find_coord(ncid, dimid = dimid, varid = varid, std_name = "latitude")
  call nf95_inquire_dimension(ncid, dimid, nclen = nlat)
  call nf95_get_var(ncid, varid, lat_min)
  call nf95_get_var(ncid, varid, lat_max, start = [nlat])

  call assert(lon_max > lon_min .and. lat_max > lat_min, &
       "test_set_all_outerm coordinate order")
  step = [(lon_max - lon_min) / (nlon - 1), (lat_max - lat_min) / (nlat - 1)] &
       * deg_to_rad

  ! As we are requiring the grid spacing to be uniform, the value of
  ! "periodic" may be deduced from the values of step(1) and nlon:
  periodic = nint(twopi / step(1)) == nlon
  print *, "periodic = ", periodic
  call assert(2 * max_radius(1) * step(1) < pi, &
       "test_set_all_outerm max_radius")

  copy = merge(max_radius(1), 0, periodic)
  allocate(ssh(1 - copy:nlon + copy, nlat))
  call get_var(periodic, max_radius(1), ssh, ncid, nlon, name = "adt", &
       new_fill_value = huge(0.))

  call nf95_close(ncid)

  call set_all_outerm(s, min_amp, max_radius, step, periodic, ssh, &
       corner = [lon_min, lat_min] * deg_to_rad, &
       min_area = pi * (min_radius * 1e3)**2)

  call shp_create_03("extremum", shpt_point, hshp_extremum)
  call dbf_add_field_03(ifield, hshp_extremum, 'ssh', ftdouble, nwidth = 13, &
       ndecimals = 6)
  call dbf_add_field_03(ifield, hshp_extremum, 'date_index', ftinteger, &
       nwidth = 4, ndecimals = 0)
  call dbf_add_field_03(ifield, hshp_extremum, 'eddy_index', ftinteger, &
       nwidth = 5, ndecimals = 0)
  call dbf_add_field_03(ifield, hshp_extremum, 'interpolat', ftinteger, &
       nwidth = 1, ndecimals = 0)
  call dbf_add_field_03(ifield, hshp_extremum, 'cyclone', ftinteger, &
       nwidth = 1, ndecimals = 0)
  call dbf_add_field_03(ifield, hshp_extremum, 'valid', ftinteger, &
       nwidth = 1, ndecimals = 0)

  call shp_create_03("outermost_contour", shpt_polygon, hshp_outermost)
  call dbf_add_field_03(ifield, hshp_outermost, 'r_eq_area', ftdouble, &
       nwidth = 10, ndecimals = 4)
  call dbf_add_field_03(ifield, hshp_outermost, 'ssh', ftdouble, nwidth = 13, &
       ndecimals = 6)
  call dbf_add_field_03(ifield, hshp_outermost, 'date_index', ftinteger, &
       nwidth = 4, ndecimals = 0)
  call dbf_add_field_03(ifield, hshp_outermost, 'eddy_index', ftinteger, &
       nwidth = 5, ndecimals = 0)

  do i = 1, s%number_vis_extr
     call shp_append_point_03(ishape, hshp_extremum, &
          s%list_vis(i)%coord_extr * rad_to_deg)
     call dbf_write_attribute_03(hshp_extremum, ishape, 0, &
          s%list_vis(i)%ssh_extr)
     call dbf_write_attribute_03(hshp_extremum, ishape, 1, 1)
     call dbf_write_attribute_03(hshp_extremum, ishape, 2, i)
     call dbf_write_attribute_03(hshp_extremum, ishape, 3, &
          merge(1, 0, s%list_vis(i)%interpolated))
     call dbf_write_attribute_03(hshp_extremum, ishape, 4, &
          merge(1, 0, s%list_vis(i)%cyclone))
     call dbf_write_attribute_03(hshp_extremum, ishape, 5, &
          merge(1, 0, s%list_vis(i)%valid))

     if (s%list_vis(i)%interpolated) then
        call shp_append_null_03(ishape, hshp_outermost)
     else
        if (s%list_vis(i)%out_cont%n_points == 0) then
           call shp_append_null_03(ishape, hshp_outermost)
        else
           call shp_append_object_03(ishape, hshp_outermost, shpt_polygon, &
                s%list_vis(i)%out_cont%points * rad_to_deg)
        end if
     end if

     if (s%list_vis(i)%out_cont%area >= 0) then
        call dbf_write_attribute_03(hshp_outermost, ishape, 0, &
             sqrt(s%list_vis(i)%out_cont%area / 1e6 / pi))
     else
        call dbf_write_attribute_03(hshp_outermost, ishape, 0, - 100.)
     end if
      
     call dbf_write_attribute_03(hshp_outermost, ishape, 1, &
          s%list_vis(i)%out_cont%ssh)
     call dbf_write_attribute_03(hshp_outermost, ishape, 2, 1)
     call dbf_write_attribute_03(hshp_outermost, ishape, 3, i)
  end do

  print *, "s%number_vis_extr = ", s%number_vis_extr

  CALL shpclose(hshp_extremum)
  print *, 'Created shapefile "extremum".'
  CALL shpclose(hshp_outermost)
  print *, 'Created shapefile "outermost_contour".'
  print *, "Average number of points per outermost contour: ", &
       sum(s%list_vis%out_cont%n_points) / real(s%number_vis_extr)

end program test_set_all_outerm
