program test_successive_overlap

  use, intrinsic:: ISO_FORTRAN_ENV

  ! Libraries:
  use jumble, only: new_unit, get_command_arg_dyn
  use nr_util, only: deg_to_rad, assert
  use shapelib, only: shpfileobject, shpclose
  use shapelib_03, only: shp_open_03

  use derived_types, only: snapshot
  use read_field_indices_m, only: read_field_indices
  use read_snapshot_m, only: read_snapshot
  use successive_overlap_m, only: successive_overlap

  implicit none

  character(len = :), allocatable:: snapshot_1, snapshot_2
  integer unit_edgelist, i, k
  type(snapshot) flow(2)
  TYPE(shpfileobject) hshp_extremum ! shapefile extremum
  TYPE(shpfileobject) hshp_outermost ! shapefile outermost_contour
  TYPE(shpfileobject) hshp_max_speed ! shapefile max_speed_contour

  real:: corner_deg(2) = [huge(0.), huge(0.)], corner(2)
  ! longitude and latitude of the corner of the whole grid, in degrees
  ! and in rad

  real:: step_deg(2) = [0.25, 0.25], step(2)
  ! longitude and latitude steps, in degrees and in rad

  logical periodic ! grid is periodic in longitude
  integer:: nlon = - 1, nlat = - 1

  integer:: dist_lim = 12
  ! We look for an overlapping eddy at dist_lim (in grid points) of
  ! the first extremum.

  namelist /main_nml/ corner_deg, step_deg, nlon, nlat, dist_lim

  !-------------------------------------------------------------------------

  call assert(command_argument_count() == 2, &
       "Required arguments: snapshot-directory snapshot-directory")
  call get_command_arg_dyn(1, snapshot_1)
  call get_command_arg_dyn(2, snapshot_2)

  write(unit = error_unit, nml = main_nml)
  write(unit = error_unit, fmt = *) "Enter namelist main_nml."
  read(unit = *, nml = main_nml)
  write(unit = *, nml = main_nml)

  ! As we are requiring the grid spacing to be uniform, the value of
  ! "periodic" may be deduced from the values of step_deg(1) and nlon:
  periodic = nint(360. / step_deg(1)) == nlon
  print *, "periodic = ", periodic
  if (periodic) call assert(2 * dist_lim * step_deg(1) < 180., &
       "test_successive_overlap dist_lim")

  corner = corner_deg * deg_to_rad
  step = step_deg * deg_to_rad

  call shp_open_03(trim(snapshot_1) // "/extremum", pszaccess = "rb", &
       hshp = hshp_extremum)
  call shp_open_03(trim(snapshot_1) // "/outermost_contour", pszaccess = "rb", &
       hshp = hshp_outermost)
  call shp_open_03(trim(snapshot_1) // "/max_speed_contour", pszaccess = "rb", &
       hshp = hshp_max_speed)
  call read_field_indices(hshp_extremum, hshp_outermost, hshp_max_speed)
  call read_snapshot(flow(1), k, hshp_extremum, hshp_outermost, &
       hshp_max_speed, corner, step, nlon, nlat, periodic, dist_lim)
  CALL shpclose(hshp_extremum)
  CALL shpclose(hshp_outermost)
  CALL shpclose(hshp_max_speed)
  print *, snapshot_1, ", k = ", k

  call shp_open_03(trim(snapshot_2) // "/extremum", pszaccess = "rb", &
       hshp = hshp_extremum)
  call shp_open_03(trim(snapshot_2) // "/outermost_contour", pszaccess = "rb", &
       hshp = hshp_outermost)
  call shp_open_03(trim(snapshot_2) // "/max_speed_contour", pszaccess = "rb", &
       hshp = hshp_max_speed)
  ! We assume that the field indices are the same than in snapshot_1.
  call read_snapshot(flow(2), k, hshp_extremum, hshp_outermost, &
       hshp_max_speed, corner, step, nlon, nlat, periodic, dist_lim)
  CALL shpclose(hshp_extremum)
  CALL shpclose(hshp_outermost)
  CALL shpclose(hshp_max_speed)
  print *, snapshot_2, ", k = ", k

  call new_unit(unit_edgelist)
  open(unit_edgelist, file = "edgelist.csv", status = "replace", &
       action  = "write")

  ! Title lines:
  write(unit_edgelist, fmt = "(1x, a)") '"predecessor date subscript" ' &
       // '"predecessor eddy subscript" "successor date subscript" ' &
       // '"successor eddy subscript"'
  write(unit_edgelist, fmt = *) "k1 i1 k2 i2 weight"

  call successive_overlap(flow, unit_edgelist, nlon, nlat, periodic, dist_lim, &
       j = 2, k = k)
  close(unit_edgelist)
  print *, 'Created file "edgelist.csv".'

  print *, snapshot_1, ":"
  print *, "Eddies for which delta_out == 1:"

  do i = 1, flow(1)%number_vis_extr
     if (flow(1)%list_vis(i)%delta_out /= huge(0)) &
          write(unit = *, fmt = "(i0, 1x)",  advance = "no") i
  end do

  print *
  print *, "Valid isolated eddies:"
  do i = 1, flow(1)%number_vis_extr
     if (flow(1)%list_vis(i)%valid .and. flow(1)%list_vis(i)%delta_out &
          == huge(0)) write(unit = *, fmt = "(i0, 1x)", advance = "no") i
  end do

  print *
  print *, Snapshot_2, ":"
  print *, "Eddies for which delta_in == 1:"

  do i = 1, flow(2)%number_vis_extr
     if (flow(2)%list_vis(i)%delta_in /= huge(0)) &
          write(unit = *, fmt = "(i0, 1x)", advance = "no") i
  end do

  print *
  print *, "Valid isolated eddies:"

  do i = 1, flow(2)%number_vis_extr
     if (flow(2)%list_vis(i)%valid .and. flow(2)%list_vis(i)%delta_in &
          == huge(0)) write(unit = *, fmt = "(i0, 1x)", advance = "no") i
  end do
  print *

end program test_successive_overlap
