module get_snapshot_m

  implicit none

contains

  subroutine get_snapshot(s, m, n_proc, k_end, max_delta, nlon, nlat, k, &
       corner, step, periodic)

    use, intrinsic:: ieee_arithmetic, only: ieee_value, IEEE_QUIET_NAN

    ! Libraries:
    use shapelib, only: shpfileobject, shpclose
    use shapelib_03, only: shp_open_03

    use derived_types, only: snapshot
    use read_snapshot_m, only: read_snapshot
    use receive_snapshot_m, only: receive_snapshot

    type(snapshot), intent(out):: s

    integer, intent(in):: m ! rank of MPI process
    integer, intent(in):: n_proc ! number of MPI processes
    integer, intent(in):: k_end ! last date index analyzed by this MPI process

    integer, intent(in):: max_delta
    ! maximum interval of date indices between which we look for
    ! overlapping of eddies

    integer, intent(in):: nlon, nlat
    integer, intent(in):: k ! date index

    real, intent(in):: corner(:) ! (2) longitude and latitude of the
    ! corner of the whole grid, in rad

    real, intent(in):: step(:) ! (2) longitude and latitude steps, in rad
    integer, intent(in):: nlon, nlat

    logical, intent(in):: periodic ! grid is periodic in longitude

    ! Local:
    TYPE(shpfileobject) hshp_extremum ! shapefile extremum_1
    TYPE(shpfileobject) hshp_outermost ! shapefile outermost_contour_1
    TYPE(shpfileobject) hshp_max_speed ! shapefile max_speed_contour_1
    integer k2

    !--------------------------------------------------------------

    if (m == n_proc .or. k <= k_end - max_delta) then
       ! Read at date k:
       call shp_open_03("extremum", pszaccess = "rb", hshp = hshp_extremum)
       call shp_open_03("outermost_contour", pszaccess = "rb", &
            hshp = hshp_outermost)
       call shp_open_03("max_speed_contour", pszaccess = "rb", &
            hshp = hshp_max_speed)
       call read_snapshot(s, k2, hshp_extremum, hshp_outermost, &
            hshp_max_speed, corner, step, nlon, nlat, periodic)
       CALL shpclose(hshp_extremum)
       CALL shpclose(hshp_outermost)
       CALL shpclose(hshp_max_speed)
    else
       call receive_snapshot(s, source = m + 1, tag = k)
    end if

  end subroutine get_snapshot

end module get_snapshot_m
