module read_snapshot_m

  implicit none

contains

  subroutine read_snapshot(s, k, hshp_extremum, hshp_outermost, &
       hshp_max_speed, corner, step, nlon, nlat, periodic, dist_lim)

    ! Note: read_field_indices must have been called before read_snapshot.

    ! Libraries:
    use contour_531, only: convert_to_ind 
    use nr_util, only: assert, deg_to_rad
    use shapelib, only: shpfileobject
    use shapelib_03, only: shp_get_info_03

    use derived_types, only: snapshot, eddy
    use read_eddy_m, only: read_eddy

    type(snapshot), intent(out):: s ! completely defined
    integer, intent(out):: k ! date index
    TYPE(shpfileobject), intent(inout):: hshp_extremum ! shapefile extremum

    TYPE(shpfileobject), intent(inout):: hshp_outermost
    ! shapefile outermost_contour

    TYPE(shpfileobject), intent(inout):: hshp_max_speed
    ! shapefile max_speed_contour

    real, intent(in):: corner(:) ! (2) longitude and latitude of the
    ! corner of the whole grid, in rad

    real, intent(in):: step(:) ! (2) longitude and latitude steps, in rad
    integer, intent(in):: nlon, nlat
    logical, intent(in):: periodic ! grid is periodic in longitude

    integer, intent(in):: dist_lim
    ! We look for an overlapping eddy at dist_lim (in grid points) of
    ! the first extremum.

    ! Local:
    integer ishape, copy
    integer k2 ! date index
    integer i ! eddy index
    type(eddy) e

    !---------------------------------------------------------------------

    call shp_get_info_03(hshp_extremum, n_entities = s%number_vis_extr)
    allocate(s%list_vis(s%number_vis_extr))

    ! The first shape gives the date index:
    call read_eddy(e, k, i, hshp_extremum, hshp_outermost, hshp_max_speed, &
         ishape = 0)
    s%list_vis(i) = e

    do ishape = 1, s%number_vis_extr - 1
       call read_eddy(e, k2, i, hshp_extremum, hshp_outermost, hshp_max_speed, &
            ishape)
       call assert(k2 == k, "read_snapshot: date index")
       s%list_vis(i) = e
    end do

    allocate(s%ind_extr(2, s%number_vis_extr))
    forall (i = 1:s%number_vis_extr) s%ind_extr(:, i) &
         = nint(convert_to_ind(s%list_vis(i)%coord_extr, corner, step))

    s%number_eddies = s%number_vis_extr
    
   ! Define s%extr_map from s%ind_extr:
    
    copy = merge(dist_lim, 0, periodic)
    allocate(s%extr_map(1 - copy:nlon + copy, nlat))
    s%extr_map(1:nlon, :) = 0

    ! Since we are not sure that there is no duplicate tuple in
    ! s%ind_extr, we must use a do construct and not a forall:
    do i = 1, s%number_vis_extr
       s%extr_map(s%ind_extr(1, i), s%ind_extr(2, i)) = i
    end do

    call assert(count(s%extr_map(1:nlon, :) /= 0) == s%number_vis_extr, &
         "read_snapshot: one eddy per grid point")

    ! Extend in longitude:
    s%extr_map(1 - copy:0, :) = s%extr_map(nlon + 1 - copy:nlon, :)
    s%extr_map(nlon + 1:nlon + copy, :) = s%extr_map(1:copy, :)

  end subroutine read_snapshot
  
end module read_snapshot_m
